local L = AceLibrary("AceLocale-2.2"):new("MageAnnounce")
local waterfall = AceLibrary("Waterfall-1.0")
local GameVersion, _, _ = GetBuildInfo()
local db

MageAnnounce = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceEvent-2.0", "AceDB-2.0")

------------ Main Code ------------
local isInParty, isInRaid
local WarnNum = 1

function MageAnnounce:OnInitialize()
	------------ Slash Commands ------------
	self.options = { 
		type = "group",
		desc = L["Options"],
		args = {
			menu = {
				type = "execute",
				name = L["Options Menu"],
				desc = L["Shows the Options Menu"],
				wfHidden = true,
				func = function()
						waterfall:Open("MageAnnounce")
					end
			},
			bgenable = {
				type = "toggle",
				name = L["Enable in BGs"],
				desc = L["Enable messages in Battlegrounds"],
				get = function() return db.showInBGs end,
				set = "ToggleShowInBGs",
			},
			customchan = {
				type = "text",
				name = L["Custom Channel"],
				desc = L["Custom channel to show messages in"],
				usage = L["<Channel>"],
				get = function() return db.customChan end,
				set = "SetCustomChan",
			},
			spellops = {
				type = "group",
				name = L["Spell Options"],
				order = 1,
				desc = L["Show Spell Options"],
				args = {
					polymsg = { -- Options for Polymorph
						type = "text",
						name = L["Poly Message"],
						desc = L["Message to be displayed when polymorphing"],
						usage = L["<Your message>"],
						order = 1,
						get = function() return db.polyMessage end,
						set = "SetpolyMessage",
					},
					polyparty = { 
						type = "toggle",
						name = L["Show Polymorph Messages in Party"],
						desc = L["Toggles the display of polymorph messages in party chat"],
						order = 2,
						get = function() return db.showPolyInParty end,
						set = "ToggleShowPolyInParty",
					},
					polyraid = {
						type = "toggle",
						name = L["Show Polymorph Messages in Raid"],
						desc = L["Toggles the display of polymorph messages in raid chat"],
						order = 3,
						get = function() return db.showPolyInRaid end,
						set = "ToggleShowPolyInRaid",
					},
					polycust = {
						type = "toggle",
						name = L["Show Polymorph Messages in Custom Channel"],
						desc = L["Toggles the display of polymorph messages in the custom channel"],
						order = 4,
						get = function() return db.showPolyInCust end,
						set = "ToggleShowPolyInCust",
					},
					csmsg = { -- Options for Counter Spell
						type = "text",
						name = L["Counterspell Message"],
						desc = L["Message to be displayed when counterspelling"],
						usage = L["<Your message>"],
						order = 5,
						get = function() return db.csMessage end,
						set = "SetcsMessage",
					},
					csparty = { 
						type = "toggle",
						name = L["Show Counterspell Messages in Party"],
						desc = L["Toggles the display of counterspell messages in party chat"],
						order = 6,
						get = function() return db.showCSInParty end,
						set = "ToggleShowCSInParty",
					},
					csraid = {
						type = "toggle",
						name = L["Show Counterspell Messages in Raid"],
						desc = L["Toggles the display of counterspell messages in raid chat"],
						order = 7,
						get = function() return db.showCSInRaid end,
						set = "ToggleShowCSInRaid",
					},
					cscust = {
						type = "toggle",
						name = L["Show Counterspell Messages in Custom Channel"],
						desc = L["Toggles the display of counterspell messages in the custom channel"],
						order = 8,
						get = function() return db.showCSInCust end,
						set = "ToggleShowCSInCust",
					},
					slowmsg = { -- Options for Slow
						type = "text",
						name = L["Slow Message"],
						desc = L["Message to be displayed when slowing"],
						usage = L["<Your message>"],
						order = 9,
						get = function() return db.slowMessage end,
						set = "SetslowMessage",
					},
					slowparty = { 
						type = "toggle",
						name = L["Show Slow Messages in Party"],
						desc = L["Toggles the display of slow messages in party chat"],
						order = 10,
						get = function() return db.showSlowInParty end,
						set = "ToggleShowSlowInParty",
					},
					slowraid = {
						type = "toggle",
						name = L["Show Slow Messages in Raid"],
						desc = L["Toggles the display of slow messages in raid chat"],
						order = 11,
						get = function() return db.showSlowInRaid end,
						set = "ToggleShowSlowInRaid",
					},
					slowcust = {
						type = "toggle",
						name = L["Show Slow Messages in Custom Channel"],
						desc = L["Toggles the display of slow messages in the custom channel"],
						order = 12,
						get = function() return db.showSlowInCust end,
						set = "ToggleShowSlowInCust",
					},
				},
			},
			portops = {
				type = "group",
				name = L["Portal Options"],
				desc = L["Show Portal Options"],
				order = 2,
				args = {
					portmsg = { 
						type = "text",
						name = L["Portal Message"],
						desc = L["Message to be displayed when opening a portal, %s will be replace with the city name"],
						usage = L["<Your message>"],
						order = 1,
						get = function() return db.portMessage end,
						set = "SetPortMessage",
					},
					portparty = {
						type = "toggle",
						name = L["Show Portal Messages in Party"],
						desc = L["Toggles the display of portal messages in party chat"],
						order = 2,
						get = function() return db.showPortInParty end,
						set = "ToggleShowPortInParty",
					},
					portraid = {
						type = "toggle",
						name = L["Show Portal Messages in Raid"],
						desc = L["Toggles the display of portal messages in raid chat"],
						order = 3,
						get = function() return db.showPortInRaid end,
						set = "ToggleShowPortInRaid",
					},
					portcust = {
						type = "toggle",
						name = L["Show Portal Messages in Custom Channel"],
						desc = L["Toggles the display of portal messages in the custom channel"],
						order = 4,
						get = function() return db.showPortInCust end,
						set = "ToggleShowPortInCust",
					},
				},
			},	
		},
	}
	MageAnnounce:RegisterChatCommand(L["Slash-Commands"], self.options)
	------------ End Slash Commands ------------
	------------ Defaults ------------
	MageAnnounce:RegisterDB("MageAnnounceDB", "MageAnnounceDBPC")
	MageAnnounce:RegisterDefaults("profile", {
	polyMessage = L["Now Polymorphing %t"],
	csMessage = L["Counterspelling %t"],
	slowMessage = L["Slowing %t"],
	portMessage = L["Incoming Portal to %s"], 
	customChan = "",
	showInBGs = true,
	showPolyInRaid = true, -- Polymorph
	showPolyInParty = true,
	showPolyInCust = true,
	showCSInRaid = true, -- Counterspell
	showCSInParty = true,
	showCSInCust = true,
	showSlowInRaid = true, -- Slow
	showSlowInParty = true,
	showSlowInCust = true,
	showPortInParty = true, -- Portals
	showPortInRaid = true,
	showPortInCust = true,
	} )
	------------ End Defaults ------------
	db = self.db.profile
end

function MageAnnounce:OnEnable()
	waterfall:Register("MageAnnounce", "aceOptions", self.options, "title","MageAnnounce Options")
	self:RegisterEvent("UNIT_SPELLCAST_START")
	self:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")
	self:RegisterEvent("PARTY_MEMBERS_CHANGED")
	self:RegisterEvent("RAID_ROSTER_UPDATE")
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA")
	MageAnnounce:PARTY_MEMBERS_CHANGED()
	MageAnnounce:RAID_ROSTER_UPDATE()
end

function MageAnnounce:PARTY_MEMBERS_CHANGED() -- Check to see if in party
	if GetNumPartyMembers() > 0 then
		isInParty = true
	else
		isInParty = false
	end
end
function MageAnnounce:RAID_ROSTER_UPDATE() -- Check to see if in raid
	if GetNumRaidMembers() > 0 then
		isInRaid = true
	else 
		isInRaid = false
	end
end

function MageAnnounce:ZONE_CHANGED_NEW_AREA()
	local Zone = GetRealZoneText()
	if Zone == ("Warsong Gulch" or "Alterac Valley" or "Arathi Basin" or "Eye of the Storm") then
		InBG = true
	else 
		InBG = false
	end
end

function MageAnnounce:UNIT_SPELLCAST_START(caster)
	if caster ~= "player" then 
		return 
	end
	local spellCasting, _, _, _, _, _ = UnitCastingInfo("player")
	if db.customChan then
		chanid, _ = GetChannelName(db.customChan)
	end
	if spellCasting then
		if strfind (spellCasting, L["Polymorph"]) then
			if isInParty and not isInRaid and db.showPolyInParty then -- Polymorph Party Messages
				SendChatMessage(db.polyMessage , "PARTY", nil, ""); 

			end
			if isInRaid and db.showPolyInRaid and WarnNum == 1 then -- Polymorph Raid Messages
				SendChatMessage(db.polyMessage , "RAID", nil, "");
				WarnNum = WarnNum + 1
			elseif WarnNum >= 2 then
				WarnNum = 1
			end
			if db.showPolyInCust and (chanid > 0) then -- Custom Channel Message
				SendChatMessage(db.polyMessage , "CHANNEL", nil, chanid);
			end
			if IsInBG and db.showInBGs then -- Polymorph BG Messages
				SendChatMessage(db.polyMessage , "BATTLEGROUND", nil, ""); 

			end
		end
		if strfind(spellCasting, L["Portal"]) then
			city = gsub (spellCasting, L["Portal"]..": ", "")
			message = gsub (db.portMessage, "%%s", city)
			if isInParty and not isInRaid and db.showPortInRaid then
				SendChatMessage(message, "PARTY", nil, ""); 
			end
			if isInRaid and db.showPortInRaid and WarnNum == 1 then
				SendChatMessage(message, "RAID", nil, ""); 
				WarnNum = WarnNum + 1
			elseif WarnNum >= 2 then
				WarnNum = 1
			end
			if db.showPortInCust and (chanid > 0) then -- Custom Channel Message
				SendChatMessage(message , "CHANNEL", nil, chanid);
			end
			if IsInBG and db.showInBGs then
				SendChatMessage(message, "BATTLEGROUND", nil, ""); 
			end
		end
	end
end
function MageAnnounce:UNIT_SPELLCAST_SUCCEEDED(caster, SpellName)
	if caster ~= "player" then 
		return 
	end
	if strfind (SpellName, L["Counterspell"]) then
		if isInParty and not isInRaid and db.showCSInParty then -- Counterspell Party Messages
			SendChatMessage(db.csMessage , "PARTY", nil, ""); 
		end
		if isInRaid and db.showCSInRaid then -- Counterspell Raid Messages
			SendChatMessage(db.csMessage , "RAID", nil, "");
		end
		if db.showCSInCust and (chanid > 0) then -- Custom Channel Message
			SendChatMessage(db.csMessage , "CHANNEL", nil, chanid);
		end
		if IsInBG and db.showInBGs then -- Slow BG Messages
			SendChatMessage(db.csMessage , "BATTLEGROUND", nil, "");
		end
	end
	if SpellName == L["Slow"] then
		if isInParty and not isInRaid and db.showSlowInParty then -- Slow Party Messages
			SendChatMessage(db.slowMessage , "PARTY", nil, ""); 

		end
		if isInRaid and db.showSlowInRaid then -- Slow Raid Messages
			SendChatMessage(db.slowMessage , "RAID", nil, "");
		end
		if db.showSlowInCust and (chanid > 0) then -- Custom Channel Message
			SendChatMessage(db.slowMessage , "CHANNEL", nil, chanid);
		end
		if IsInBG and db.showInBGs then -- Slow BG Messages
			SendChatMessage(db.slowMessage , "BATTLEGROUND", nil, "");
		end
	end
end
------------ End Main Code ------------

------------ Set Messages ------------
function MageAnnounce:SetpolyMessage(newValue) -- Set Polymorph Message
	db.polyMessage = newValue
end
function MageAnnounce:SetcsMessage(newValue) -- Set Counterspell Message
	db.csMessage = newValue
end
function MageAnnounce:SetslowMessage(newValue) -- Set Slow Message
	db.slowMessage = newValue
end
function MageAnnounce:SetPortMessage(newValue) -- Set Shattrath Portal Message
	db.portMessage = newValue
end
function MageAnnounce:SetCustomChan(newValue) -- Set Custom Channel
	local id, _ = GetChannelName(newValue)
	if newValue and id == 0 then
		JoinChannelByName(newValue)
		ChatFrame_AddChannel(DEFAULT_CHAT_FRAME, newValue)
	end
	db.customChan = newValue
end
------------ End Set Messages ------------

------------ Set Options ------------
function MageAnnounce:ToggleShowInBGs() -- Messages in BGs Toggle
	db.showInBGs = not db.showInBGs
end
---------------------------------------------
function MageAnnounce:ToggleShowPolyInParty() -- Polymorph in Party Message Toggle
	db.showPolyInParty = not db.showPolyInParty
end
function MageAnnounce:ToggleShowPolyInRaid() -- Polymorph in Raid Message Toggle
	db.showPolyInRaid = not db.showPolyInRaid
end
function MageAnnounce:ToggleShowPolyInCust() -- Polymorph in Custom Channel Toggle
	db.showPolyInCust = not db.showPolyInCust
end
---------------------------------------------
function MageAnnounce:ToggleShowCSInParty() -- Counterspell in Party Message Toggle
	db.showCSInParty = not db.showCSInParty
end
function MageAnnounce:ToggleShowCSInRaid() -- Counterspell in Raid Message Toggle
	db.showCSInRaid = not db.showCSInRaid
end
function MageAnnounce:ToggleShowCSInCust() -- Counterspell in Custom Channel Toggle
	db.showCSInCust = not db.showCSInCust
end
---------------------------------------------
function MageAnnounce:ToggleShowSlowInParty() -- Slow in Party Message Toggle
	db.showSlowInParty = not db.showSlowInParty
end
function MageAnnounce:ToggleShowSlowInRaid() -- Slow in Raid Message Toggle
	db.showSlowInRaid = not db.showSlowInRaid
end
function MageAnnounce:ToggleShowSlowInCust() -- Slow in Custom Channel Toggle
	db.showSlowInCust = not db.showSlowInCust
end
---------------------------------------------
function MageAnnounce:ToggleShowPortInParty() -- Portal in Party Message Toggle
	db.showPortInParty = not db.showPortInParty
end
function MageAnnounce:ToggleShowPortInRaid() -- Portal in Raid Message Toggle
	db.showPortInRaid = not db.showPortInRaid
end
function MageAnnounce:ToggleShowPortInCust() -- Portal in Custom Channel Toggle
	db.showPortInCust = not db.showPortInCust
end
------------ End Set Options ------------